// $Id: MusicConfiguration.java,v 1.5 2004/05/20 14:55:47 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.io.*;

public class MusicConfiguration implements ActionListener {
	private JFrame confFrame;
	private JTextField host, newdir;
	private JComboBox deviceList;
	private Object selectedDevice;
	private String selectedHost;
	private MusicList ml;
	private Vector volumes, rowData;
	private JScrollPane scrollPane;
	private JTable table;
	private boolean volumesChanged;

	public MusicConfiguration () {
		confFrame = new JFrame ("MusicControl configuration");

		JPanel contentPanel = new JPanel(new BorderLayout());
		confFrame.getContentPane().add(contentPanel);

		String[] devices = {"OSS",
							"ESD"};

		JPanel dPanel = new JPanel(new BorderLayout());
		dPanel.add(new JLabel("Audio device"), BorderLayout.NORTH);
		JPanel d2Panel = new JPanel(new GridLayout(2,2));
		d2Panel.add(new JLabel("Device:"));
		deviceList = new JComboBox(devices);
		selectedDevice = deviceList.getSelectedItem();
		d2Panel.add(deviceList);
		d2Panel.add(new JLabel("Host:"));
		host = new JTextField("");
		selectedHost = "";
		d2Panel.add(host);

		dPanel.add(d2Panel, BorderLayout.CENTER);
		contentPanel.add(dPanel, BorderLayout.NORTH);

		JPanel vPanel = new JPanel(new BorderLayout());
		vPanel.add(new JLabel("Music volumes"), BorderLayout.NORTH);

		volumes = new Vector();
		rowData = new Vector();
		Vector colData = new Vector();
		colData.add((Object)"Directory");
		table = new JTable(new SelectOnlyTableModel(rowData, colData));
		scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(new Dimension(200, 100));
		vPanel.add(scrollPane, BorderLayout.CENTER);

		JPanel v2Panel = new JPanel(new GridLayout(2,2));
		JButton addButton = new JButton("Add");
		addButton.addActionListener(this);
		v2Panel.add(addButton);
		newdir = new JTextField("");
		v2Panel.add(newdir);
		JButton removeButton = new JButton("Remove");
		removeButton.addActionListener(this);
		v2Panel.add(removeButton);
		vPanel.add(v2Panel, BorderLayout.SOUTH);

		contentPanel.add(vPanel, BorderLayout.CENTER);

		volumesChanged = false;

		JPanel bPanel = new JPanel(new GridLayout(1,2));
		JButton okButton = new JButton("Ok");
		okButton.addActionListener(this);
		bPanel.add(okButton);
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		bPanel.add(cancelButton);
		contentPanel.add(bPanel, BorderLayout.SOUTH);

		confFrame.pack();
	}

	public String getDevice () {
		return (String)selectedDevice;
	}

	public String getHost () {
		return selectedHost;
	}

	public void setMusicList (MusicList ml) {
		this.ml = ml;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("Configuration...")) {
			if (selectedDevice != null) {
				deviceList.setSelectedItem(selectedDevice);
			}
			host.setText(selectedHost);
			volumes = ml.getVolumes();

			Vector v;
			MusicVolume mv;
			rowData.removeAllElements();
			for (int i = 0; i < volumes.size(); ++i) {
				mv = (MusicVolume)volumes.get(i);
				v = new Vector();
				v.add(mv.getDirectory());
				rowData.add(v);
			}
			TableModel model = table.getModel();
			TableModelEvent tme = new TableModelEvent(model);
			table.tableChanged(tme);
			volumesChanged = false;

			confFrame.setVisible(true);
		} else if (command.equals("Ok")) {
			selectedDevice = deviceList.getSelectedItem();
			selectedHost = host.getText();

			if (volumesChanged) {
				volumes.removeAllElements();
				for (int i = 0; i < rowData.size(); ++i) {
					Vector v = (Vector)rowData.get(i);
					volumes.add(new MusicVolume((String)v.get(0),i+1));
				}
				ml.rescanClean();
			}

			confFrame.setVisible(false);
		} else if (command.equals("Cancel")) {
			confFrame.setVisible(false);
		} else if (command.equals("Add")) {
			File dir = new File(newdir.getText());
			if (dir.isDirectory()) {
				Vector v = new Vector();
				v.add(newdir.getText());
				rowData.add(v);
				TableModel model = table.getModel();
				TableModelEvent tme = new TableModelEvent(model);
				table.tableChanged(tme);
				volumesChanged = true;
			}
			newdir.setText("");
		} else if (command.equals("Remove")) {
			int curRow = table.getSelectedRow();
			if (curRow != -1) {
				rowData.removeElementAt(curRow);
				TableModel model = table.getModel();
				TableModelEvent tme = new TableModelEvent(model, curRow, curRow, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE);
				table.tableChanged(tme);
				volumesChanged = true;
			}
		}
	}
}
