// $Id: MusicControl.java,v 1.20 2003/12/28 01:25:39 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class MusicControl implements ActionListener,Runnable {
	protected static JFrame mainFrame;
	protected MusicPlayer[] players;
	private MusicQueue mq1;
	private MusicList ml;
	private Thread updater;
	private MusicStatus status;
	private MusicConfiguration configuration;
	private JCheckBoxMenuItem stopWhenFinishedItem;

	public MusicControl () {
		configuration = new MusicConfiguration();
		players = new MusicPlayer[3];
		players[0] = new OggPlayer(configuration);
		players[1] = new Mp3Player(configuration);
		players[2] = new ModPlayer(configuration);
	}

	public static void main (String[] args) {
		MusicControl mc = new MusicControl();
		mc.setupGUI(args);
	}

	protected void setupGUI (String[] dirs) {
		mainFrame = new JFrame ("MusicControl");

		JMenuBar menuBar = new JMenuBar();
        menuBar.setOpaque(true);

		JMenu fileMenu = new JMenu("File");
		JMenuItem confItem = new JMenuItem("Configuration...");
		confItem.addActionListener(configuration);
		fileMenu.add(confItem);
		JMenuItem reloadItem = new JMenuItem("Reload volumes");
		reloadItem.addActionListener(this);
		fileMenu.add(reloadItem);
		JMenuItem exitItem = new JMenuItem("Exit");
		exitItem.addActionListener(this);
		fileMenu.add(exitItem);

		JMenu controlsMenu = new JMenu("Controls");
		stopWhenFinishedItem = new JCheckBoxMenuItem("Stop when finished", false);
		stopWhenFinishedItem.addActionListener(this);
		controlsMenu.add(stopWhenFinishedItem);
		controlsMenu.addSeparator();
		JMenuItem stopAndRequeueItem = new JMenuItem("Stop and requeue");
		stopAndRequeueItem.addActionListener(this);
		controlsMenu.add(stopAndRequeueItem);

		JMenu helpMenu = new JMenu("Help");
		JMenuItem aboutItem = new JMenuItem("About");
		aboutItem.addActionListener(this);
		helpMenu.add(aboutItem);

		menuBar.add(fileMenu);
		menuBar.add(controlsMenu);
		menuBar.add(Box.createHorizontalGlue());
		menuBar.add(helpMenu);

		mainFrame.setJMenuBar(menuBar);

		JPanel contentPanel1 = new JPanel(new GridLayout(1,3));
		JPanel contentPanel2 = new JPanel(new BorderLayout());

		JSplitPane mainSP = new JSplitPane (JSplitPane.VERTICAL_SPLIT,contentPanel1,contentPanel2);
		mainFrame.getContentPane().add(mainSP);

		mq1 = new MusicQueue("Random queue");
		Component q1c = mq1.getComponent();
		contentPanel1.add(q1c);

		MusicQueue mq2 = new MusicQueue("Manual queue");
		Component q2c = mq2.getComponent();
		contentPanel1.add(q2c);

		status = new MusicStatus(mq2, mq1, players, this);
		Component statusc = status.getComponent();
		contentPanel1.add(statusc);

		ml = new MusicList(mq2,dirs);
		configuration.setMusicList(ml);
		Component mlc = ml.getComponent();
		contentPanel2.add(mlc,BorderLayout.CENTER);

		updater = new Thread(this);
		updater.start();

		mainFrame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					status.stopPlaying();
					System.exit(0);
				}
			});

		mainFrame.pack();
		mainFrame.setVisible(true);
	}

	public void musicStopped () {
		stopWhenFinishedItem.setSelected(false);
		status.stopWhenFinished(stopWhenFinishedItem.isSelected());
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("Exit")) {
			status.stopPlaying();
			System.exit(0);
		} else if (command.equals("Reload volumes")) {
			ml.rescan();
		} else if (command.equals("About")) {
		} else if (command.equals("Stop when finished")) {
			status.stopWhenFinished(stopWhenFinishedItem.isSelected());
		} else if (command.equals("Stop and requeue")) {
			status.stopAndRequeue();
		}
	}

	public void run () {
		while (true) {
			int i = mq1.size();
			while (i < 5) {
				Song s = ml.pickRandom();
				if (s != null) {
					mq1.addSong(s);
				}
				++i;
			}
			try {
				Thread.sleep(5000);
			} catch (Exception e) {
			}
		}
	}
}
