// $Id: MusicList.java,v 1.15 2004/05/20 14:55:47 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;
import java.util.Random;

public class MusicList implements ActionListener, MouseListener {
	private JPanel panel;
	private MusicQueue queue;
	private JScrollPane scrollPane;
	private JButton enqueue, search;
	private Vector rowData;
	private Vector songs;
	private JTable table;
	private Vector volumes;
	private Random random;

	public MusicList (MusicQueue q, String[] dirs) {
		queue = q;

		rowData = new Vector();
		songs = new Vector();

		Vector colData = new Vector();
		colData.add((Object)"Directory");
		colData.add((Object)"Filename");

		table = new JTable(new SelectOnlyTableModel(rowData, colData));
		table.addMouseListener(this);
		scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(new Dimension(200, 100));
		panel = new JPanel(new BorderLayout());
		panel.add(new JLabel("Available music"), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);

		JPanel actionPanel = new JPanel(new GridLayout(1,3));
		enqueue = new JButton("Enqueue");
		enqueue.addActionListener(this);
		search = new JButton("Search");
		search.addActionListener(this);
		actionPanel.add(enqueue);
		actionPanel.add(search);
		panel.add(actionPanel, BorderLayout.SOUTH);

		volumes = new Vector();

		for (int i = 0; i < dirs.length; ++i) {
			volumes.add(new MusicVolume(dirs[i], i+1));
			((MusicVolume)volumes.get(i)).rescan(songs);
		}
		syncRowData();

		random = new Random();
	}

	public Component getComponent () {
		return panel;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		int[] curRows = table.getSelectedRows();
		int i;
		if (command.equals("Enqueue")) {
			for (i = 0; i < curRows.length; ++i) {
				queue.addSong((Song)songs.get(curRows[i]));
			}
		} else if (command.equals("Search")) {
			MusicSearch search = new MusicSearch(songs,queue);
		} else {
			System.err.println("huh? " + command);
		}
	}

	public void mouseClicked(MouseEvent e)
	{
		if(e.getClickCount() == 2) {
			enqueue.doClick();
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	private void syncRowData () {
		int i;
		Vector v;
		Song s;

		rowData.removeAllElements();
		for (i = 0; i < songs.size(); ++i) {
			s = (Song)songs.get(i);
			v = new Vector();
			v.add(s.getDirectory());
			v.add(s.getFilename());
			rowData.add(v);
		}
		TableModel model = table.getModel();
		TableModelEvent tme = new TableModelEvent(model);
		table.tableChanged(tme);
	}

	public Song pickRandom () {
		if (songs.size() > 0) {
			return (Song)songs.get(random.nextInt(songs.size()));
		} else {
			return null;
		}
	}

	public void rescan () {
		int i;

		for (i = 0; i < volumes.size(); ++i) {
			((MusicVolume)volumes.get(i)).rescan(songs);
		}
		syncRowData();
	}

	public void rescanClean () {
		songs.removeAllElements();
		rescan();
	}

	public Vector getVolumes () {
		return volumes;
	}
}
