// $Id: MusicQueue.java,v 1.11 2004/05/20 14:55:47 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;

public class MusicQueue implements ActionListener, MouseListener  {
	private JTable table;
	private JScrollPane scrollPane;
	private JPanel panel;
	private JButton up,down,delete;
	private Vector rowData;
	private Vector songs;

	public MusicQueue (String title) {
		rowData = new Vector();
		songs = new Vector();

		Vector colData = new Vector();
		colData.add((Object)"Directory");
		colData.add((Object)"Filename");

		table = new JTable(new SelectOnlyTableModel(rowData, colData));
		table.addMouseListener(this);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(new Dimension(200, 103));
		panel = new JPanel(new BorderLayout());
		panel.add(new JLabel(title), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);

		JPanel actionPanel = new JPanel(new GridLayout(1,3));
		up = new JButton("Up");
		up.addActionListener(this);
		down = new JButton("Down");
		down.addActionListener(this);
		delete = new JButton("Delete");
		delete.addActionListener(this);
		actionPanel.add(up);
		actionPanel.add(down);
		actionPanel.add(delete);
		panel.add(actionPanel, BorderLayout.SOUTH);
	}

	public Component getComponent () {
		return panel;
	}

	public void addSong (Song song) {
		addSong(song, false);
	}

	public void addSong (Song song, boolean inFront) {
		if (inFront) {
			songs.add(0, song);
		} else {
			songs.add(song);
		}
		Vector data = new Vector();
		data.add(song.getDirectory());
		data.add(song.getFilename());
		if (inFront) {
			rowData.add(0, data);
		} else {
			rowData.add(data);
		}

		TableModel model = table.getModel();
		TableModelEvent tme = new TableModelEvent(model, rowData.size()-1, rowData.size()-1, TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT);
		table.tableChanged(tme);
	}

	public Song getSong () {
		if (songs.size() > 0) {
			Song song = (Song)songs.get(0);
			songs.removeElementAt(0);
			rowData.removeElementAt(0);

			TableModel model = table.getModel();
			TableModelEvent tme = new TableModelEvent(model, 0, 0, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE);
			table.tableChanged(tme);

			return song;
		} else {
			return null;
		}
	}

	public int size () {
		return songs.size();
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		int curRow = table.getSelectedRow();
		if (command.equals("Up")) {
			if (curRow > 0) {
				Object obj1 = rowData.get(curRow - 1);
				Object obj2 = rowData.get(curRow);
				rowData.set(curRow - 1, obj2);
				rowData.set(curRow, obj1);

				Object song1 = songs.get(curRow - 1);
				Object song2 = songs.get(curRow);
				songs.set(curRow - 1, song2);
				songs.set(curRow, song1);

				TableModel model = table.getModel();
				TableModelEvent tme = new TableModelEvent(model, curRow - 1, curRow);
				table.tableChanged(tme);
				table.setRowSelectionInterval(curRow - 1, curRow - 1);
			}
		} else if (command.equals("Down")) {
			if (curRow != -1 && curRow < rowData.size() - 1) {
				Object obj1 = rowData.get(curRow);
				Object obj2 = rowData.get(curRow + 1);
				rowData.set(curRow, obj2);
				rowData.set(curRow + 1, obj1);

				Object song1 = songs.get(curRow);
				Object song2 = songs.get(curRow + 1);
				songs.set(curRow, song2);
				songs.set(curRow + 1, song1);

				TableModel model = table.getModel();
				TableModelEvent tme = new TableModelEvent(model, curRow + 1, curRow);
				table.tableChanged(tme);
				table.setRowSelectionInterval(curRow + 1, curRow + 1);
			}
		} else if (command.equals("Delete")) {
			if (curRow != -1) {
				rowData.removeElementAt(curRow);
				songs.removeElementAt(curRow);
				TableModel model = table.getModel();
				TableModelEvent tme = new TableModelEvent(model, curRow, curRow, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE);
				table.tableChanged(tme);				
			}
		}
	}

	public void mouseClicked(MouseEvent e)
	{
		if(e.getClickCount() == 2) {
			delete.doClick();
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
}
