// $Id: MusicSearch.java,v 1.5 2004/05/20 14:55:47 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;

public class MusicSearch implements ActionListener, MouseListener {
	private Vector songs, rowData, foundSongs;
	private JFrame searchFrame;
	private JButton enqueue;
	private JTextField searchField;
	private JTable table;
	private MusicQueue queue;

	public MusicSearch (Vector songs, MusicQueue queue) {
		this.songs = songs;
		this.queue = queue;

		searchFrame = new JFrame("Search");

		JPanel contentPanel = new JPanel(new BorderLayout());
		searchFrame.getContentPane().add(contentPanel);

		JPanel inputPanel = new JPanel(new BorderLayout());
		inputPanel.add(new JLabel(" Search for:  "),BorderLayout.WEST);
		searchField = new JTextField(50);
		searchField.addActionListener(this);
		inputPanel.add(searchField,BorderLayout.CENTER);

		contentPanel.add(inputPanel,BorderLayout.NORTH);

		rowData = new Vector();
		foundSongs = new Vector();

		Vector colData = new Vector();
		colData.add((Object)"Directory");
		colData.add((Object)"Filename");

		table = new JTable(new SelectOnlyTableModel(rowData, colData));
		table.addMouseListener(this);
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(new Dimension(200, 200));
		contentPanel.add(scrollPane, BorderLayout.CENTER);

		JPanel actionPanel = new JPanel(new GridLayout(1,2));
		enqueue = new JButton("Enqueue");
		enqueue.addActionListener(this);
		actionPanel.add(enqueue);
		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(this);
		actionPanel.add(closeButton);
		contentPanel.add(actionPanel,BorderLayout.SOUTH);

		searchFrame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					searchFrame.dispose();
				}
			});

		searchFrame.pack();
		searchFrame.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		int i;

		String command = e.getActionCommand();
		if (command.equals("Close")) {
			searchFrame.dispose();
		} else if (command.equals("Enqueue")) {
			int[] curRows = table.getSelectedRows();
			for (i = 0; i < curRows.length; ++i) {
				queue.addSong((Song)foundSongs.get(curRows[i]));
			}
		} else {
			doSearch(command);
		}
	}

	public void mouseClicked(MouseEvent e)
	{
		if(e.getClickCount() == 2) {
			enqueue.doClick();
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	private void doSearch (String str) {
		int i;
		Song song;
		Vector v;

		foundSongs.removeAllElements();
		for (i = 0; i < songs.size(); ++i) {
			song = (Song)songs.get(i);
			if(song.matchesString(str)) {
				foundSongs.add(song);
			}
		}

		rowData.removeAllElements();
		for (i = 0; i < foundSongs.size(); ++i) {
			song = (Song)foundSongs.get(i);
			v = new Vector();
			v.add(song.getDirectory());
			v.add(song.getFilename());
			rowData.add(v);
		}

		TableModel model = table.getModel();
		TableModelEvent tme = new TableModelEvent(model);
		table.tableChanged(tme);
	}
}
