// $Id: MusicStatus.java,v 1.14 2004/05/20 14:55:47 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;

public class MusicStatus implements ActionListener,Runnable {
	private JPanel panel;
	private MusicQueue queue1, queue2;
	private JButton play, stop, skip;
	private JTextField statusField, fileField, songField, posField, lengthField;
	private MusicPlayer[] players;
	private MusicPlayer player;
	private MusicControl mc;
	private Thread updater;
	private boolean skipping, stopWhenFinished;

	public MusicStatus (MusicQueue q1, MusicQueue q2, MusicPlayer[] players, MusicControl mc) {
		queue1 = q1;
		queue2 = q2;
		this.players = players;
		this.mc = mc;
		player = null;
		skipping = false;
		stopWhenFinished = false;

		panel = new JPanel(new BorderLayout());
		panel.add(new JLabel("Play status"), BorderLayout.NORTH);

		GridBagConstraints gbc = new GridBagConstraints();
		GridBagLayout gbl = new GridBagLayout();
		JPanel stPanel = new JPanel(gbl);

		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 0;
		gbc.weighty = 1;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;

		JLabel statusLabel = new JLabel("Status:");
		gbl.setConstraints(statusLabel, gbc);
		stPanel.add(statusLabel);
		gbc.gridx = 1;
		gbc.weightx = 1;
		statusField = new JTextField("Stopped");
		statusField.setEditable(false);
		gbl.setConstraints(statusField, gbc);
		stPanel.add(statusField);

		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 0;
		JLabel fileLabel = new JLabel("File:");
		gbl.setConstraints(fileLabel, gbc);
		stPanel.add(fileLabel);
		gbc.gridx = 1;
		gbc.weightx = 1;
		fileField = new JTextField("(none)");
		fileField.setEditable(false);
		gbl.setConstraints(fileField, gbc);
		stPanel.add(fileField);

		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.weightx = 0;
		JLabel songLabel = new JLabel("Song:");
		gbl.setConstraints(songLabel, gbc);
		stPanel.add(songLabel);
		gbc.gridx = 1;
		gbc.weightx = 1;
		songField = new JTextField("(none)");
		songField.setEditable(false);
		gbl.setConstraints(songField, gbc);
		stPanel.add(songField);

		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.weightx = 0;
		JLabel posLabel = new JLabel("Position:  ");
		gbl.setConstraints(posLabel, gbc);
		stPanel.add(posLabel);
		gbc.gridx = 1;
		gbc.weightx = 1;
		posField = new JTextField("N/A");
		posField.setEditable(false);
		gbl.setConstraints(posField, gbc);
		stPanel.add(posField);

		gbc.gridx = 0;
		gbc.gridy = 4;
		gbc.weightx = 0;
		JLabel lengthLabel = new JLabel("Length:");
		gbl.setConstraints(lengthLabel, gbc);
		stPanel.add(lengthLabel);
		gbc.gridx = 1;
		gbc.weightx = 1;
		lengthField = new JTextField("N/A");
		lengthField.setEditable(false);
		gbl.setConstraints(lengthField, gbc);
		stPanel.add(lengthField);

		panel.add(stPanel, BorderLayout.CENTER);

		JPanel actionPanel = new JPanel(new GridLayout(1,3));
		play = new JButton("Play");
		play.addActionListener(this);
		stop = new JButton("Stop");
		stop.addActionListener(this);
		skip = new JButton("Skip");
		skip.addActionListener(this);
		actionPanel.add(play);
		actionPanel.add(stop);
		actionPanel.add(skip);
		panel.add(actionPanel, BorderLayout.SOUTH);
		updater = new Thread(this);
		updater.start();
	}

	public Component getComponent () {
		return panel;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals("Play") || command.equals("Skip")) {
			playNext();
		} else if (command.equals("Pause")) {
			player.pauseSong();
			play.setText("Resume");
		} else if (command.equals("Resume")) {
			player.resumeSong();
			play.setText("Pause");
		} else if (command.equals("Stop")) {
			if (player != null) {
				skipping = true;
				player.stopSong();
				player = null;
				play.setText("Play");
				skipping = false;
			}
		}
	}

	public void stopPlaying () {
		if (player != null) {
			skipping = true;
			player.stopSong();
			player = null;
		}
		skipping = false;
	}

	public void stopAndRequeue () {
		if (player != null) {
			Song song = player.getSong();
			if (song != null) {
				queue1.addSong(song, true);
				stopPlaying();
				play.setText("Play");
			}
		}
	}

	private void playNext () {
		Song song;
		int i;

		if (player != null) {
			skipping = true;
			player.stopSong();
		}
		song = queue1.getSong();
		if (song == null) {
			song = queue2.getSong();
		}
		if (song != null) {
			for (i = 0; i < players.length; ++i) {
				if (players[i].acceptsType(song.getType())) {
					player = players[i];
					i = players.length;
				}
			}
			if (player != null) {
				player.playSong(song);
				play.setText("Pause");
			}
		} else {
			player = null;
			play.setText("Play");
		}
		skipping = false;
	}

	public void stopWhenFinished (boolean swf) {
		stopWhenFinished = swf;
	}

	public void run () {
		while (true) {
			//			System.err.println("Checking status");
			String windowTitle = "MusicControl";
			if (player != null) {
				String status = player.getStatus();
				statusField.setText(status);
				fileField.setText(player.getFilename());
				songField.setText(player.getTitle());
				posField.setText(player.getPosition());
				lengthField.setText(player.getLength());
				if (status.equals("Stopped") && !skipping) {
					player = null;
					if (stopWhenFinished) {
						play.setText("Play");
						mc.musicStopped();
					} else {
						playNext();
					}
				}
 				if (!skipping)
					windowTitle += " - " + player.getTitle();
			} else {
				statusField.setText("Stopped");
				songField.setText("(none)");
 				MusicControl.mainFrame.setTitle("MusicControl");
			}
 			MusicControl.mainFrame.setTitle(windowTitle);
			try {
				Thread.sleep(1000);
			} catch (Exception e) {
			}
		}
	}
}
