// $Id: OggPlayer.java,v 1.14 2003/12/28 01:18:49 xi Exp $

/******************************************************************************
*    MusicControl - An advanced music player
*    Copyright (C) 2002,2003  Christian Laursen <xi@borderworlds.dk>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

package dk.borderworlds;

import java.io.*;

public class OggPlayer extends MusicPlayer implements Runnable {
	private Song song;
	private Process proc;
	private BufferedInputStream bis;
	String name;
	private Thread inputreader;
	private String status;
	private byte[] statusLine;
	private String title, filename;
	private int pausePosition;
	private MusicConfiguration configuration;

	public OggPlayer (MusicConfiguration configuration) {
		song = null;
		proc = null;
		this.configuration = configuration;
	}

	public void playSong (Song song) {
		this.song = song;
		playSong(0);
	}

	public void playSong (int offset) {
		if (offset == 0) {
			status = "Starting";
		} else {
			status = "Resuming";
		}
		title = song.getFilename();
		filename = song.getFilename();
		Runtime rt = Runtime.getRuntime();
		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			try {
				Thread.sleep(300);
			} catch (Exception e) {
			}
			proc = null;
		}
		try {
			String device = configuration.getDevice();
			String host = configuration.getHost();
			int args = (pausePosition == 0) ? 5 : 7;
			if (device.equals("ESD")) {
				args += 2;
			}
			String[] cmdarray = new String[args];
			cmdarray[0] = "ogg123";
			cmdarray[1] = "-v";
			cmdarray[2] = "-d";
			cmdarray[3] = "oss";
			if (device.equals("ESD")) {
				cmdarray[3] = "esd";
				cmdarray[4] = "-o";
				cmdarray[5] = "host:" + host;
			}
			if (pausePosition != 0) {
				cmdarray[args-3] = "-k";
				cmdarray[args-2] = "" + pausePosition;
			}
			cmdarray[args - 1] = song.getDirectory() + File.separator + song.getFilename();
			proc = rt.exec(cmdarray);
			InputStream is = proc.getErrorStream();
			bis = new BufferedInputStream(is);
			inputreader = new Thread (this);
			inputreader.start();
			status = "Playing";
		} catch (Exception e) {
			System.err.println(e);
		}
	}

	public void pauseSong () {
		if (statusLine != null) {
			String mins = new String(statusLine, 0, 2);
			String secs = new String(statusLine, 3, 2);
			pausePosition = Integer.parseInt(mins) * 60 + Integer.parseInt(secs);
		} else {
			pausePosition = 0;
		}

		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			proc = null;
		}
	}

	public void resumeSong () {
		if (pausePosition != 0) {
			playSong(pausePosition);
			pausePosition = 0;
		}
	}

	public void stopSong () {
		if (proc != null) {
			proc.destroy();
			while (bis != null) {
				try {
					Thread.sleep(300);
				} catch (Exception e) {
				}
			}
			proc = null;
			song = null;
		}
		try {
			Thread.sleep(300);
		} catch (Exception e) {
		}
	}

	public String getStatus () {
		if (status.equals("Retrying")) {
			playSong(0);
			return "Retrying";
		}
		if (status.equals("Playing") && bis == null) {
			if (pausePosition == 0) {
				status = "Stopped";
				song = null;
			} else {
				status = "Paused";
			}
		}
		return status;
	}

	public String getFilename () {
		if (song == null) {
			return "(none)";
		} else {
			return filename;
		}
	}

	public String getTitle () {
		if (song == null) {
			return "(none)";
		} else {
			return title;
		}
	}

	public String getPosition () {
		if (statusLine != null) {
			return new String(statusLine, 0, 5);
		} else {
			return("N/A");
		}
	}

	public String getLength () {
		if (statusLine != null) {
			return new String(statusLine, 5, 5);
		} else {
			return("N/A");
		}
	}

	public boolean acceptsType (String type) {
		return type.equals("ogg");
	}

	public Song getSong () {
		return song;
	}

	public void run () {
		byte[] b, lsl;
		int i, j, k, lsll;
		boolean cont = true;

		b = new byte[1024];
		statusLine = new byte[10];
		lsl = new byte[80];
		lsll = 0;

		for (i = 0; i < 10; ++i) {
			statusLine[i] = 32;
		}

		while (cont) {
			try {
				i = bis.read(b, 0, 1024);
				if (i > 0) {
					//					System.err.print(new String (b, 0, i));
					for (j = 0; j < i; ++j) {
						if (b[j] == 10 || b[j] == 13) {
							if (lsll > 40) {
								if (lsl[0] == 'T' && lsl[1] == 'i' && lsl[2] == 'm' && lsl[3] == 'e' && lsl[4] == ':') {
									for (k = 6; k < 11; ++k) {
										statusLine[k-6] = lsl[k];
									}
									for (k = 29; k < 34; ++k) {
										statusLine[k-24] = lsl[k];
									}
								}
							}
							if (lsll > 8) {
								if (lsl[0] == 'T' && lsl[1] == 'i' && lsl[2] == 't' && lsl[3] == 'l' &&
									lsl[4] == 'e' && lsl[5] == ':') {
									title = new String(lsl, 7, lsll - 7);
								}
							}
							if (lsll == 30) {
								if (lsl[26] == 'o' && lsl[27] == 's' && lsl[28] == 's' && lsl[29] == '.') {
									status = "Retrying";
								}
							}
							lsll = 0;
						} else {
							lsl[lsll++] = b[j];
						}
					}
				} else if (i == -1) {
					bis.close();
					bis = null;
					cont = false;
				}
			} catch (Exception e) {
				if (e.getClass().getName().equals("java.io.IOException")) {
					bis = null;
					cont = false;
				}
			}
		}
	}
}
